#include "vmdevice.h"
#include <stdint.h>
#include <stdio.h>
#include <math.h>
#include "i2c.h"
#include "net.h"
#include "ds1621.h"


#define DS1621_ADDR     0x92

//float sprintf is not supported by default by SDCC
void print_float_temp(char * stxt1, const float value, char * stxt2)
{
    net_sendtxt(stxt1, DEBUG_SOCKET_NUM);
    print_float(value);
    net_sendtxt(stxt2, DEBUG_SOCKET_NUM);
}

void main(void)
{
	float ftemp;
	uint8_t config = 0;
	uint8_t sensor_initialized = 0;
	uint16_t tmp16;

	SET_I2CFREQ(100);   // set I2C SCL frequency to 100kHz
	NETTOUT = 10;       // network transmitt timeout 10 * 10 = 100ms

	ds1621_get_TH(DS1621_ADDR, &tmp16);
    print_float_temp("TH: ",uitemp_to_float(tmp16)," C\r");
    ds1621_get_TL(DS1621_ADDR, &tmp16);
    print_float_temp("TL: ",uitemp_to_float(tmp16)," C\r");

    sensor_initialized = init_ds1621(DS1621_ADDR, 23.5, 29.5);

    ds1621_get_TH(DS1621_ADDR, &tmp16);
    print_float_temp("TH: ",uitemp_to_float(tmp16)," C\r");
    ds1621_get_TL(DS1621_ADDR, &tmp16);
    print_float_temp("TH: ",uitemp_to_float(tmp16)," C\r");

    SET_TIMER(1000);
    TIMER_NONBLOCKING;
	while(1)
	{
        if (FTMR) {     // check if timer flag is set
            FTMR = 0;   // Clear timer flag
            if (sensor_initialized == 0) {
                sensor_initialized = ds1621_start_convert(DS1621_ADDR);
            } else {
                if (ds1621_read_hrtemp(DS1621_ADDR, &ftemp)) {
                    print_float_temp("T: ",ftemp," C\r");
                }

                if (ds1621_get_config(DS1621_ADDR, &config)) {
                        if(config & DS1621_THF) {
                                net_sendtxt("TH Flag\r",DEBUG_SOCKET_NUM);
                                ds1621_reset_flags(DS1621_ADDR);
                        }
                        if(config & DS1621_TLF) {
                                net_sendtxt("TL Flag\r",DEBUG_SOCKET_NUM);
                                ds1621_reset_flags(DS1621_ADDR);
                        }
                }
            }
        }
    }
}
