#include "vmdevice.h"
#include <stdint.h>
#include "i2c.h"
#include "net.h"
#include "ds1621.h"

#define DS1621_ADDR     0x92

typedef struct {
    uint8_t   cmd;
    FL_INT_T  temp;
    uint8_t   config;
} DATA_PACKET_T;

void main(void)
{
    uint8_t cmd;
	FL_INT_T * ptemp;
	uint8_t config = 0;
	uint8_t sensor_initialized = 0;
	uint16_t tmp16;

	DATA_PACKET_T packet;

	SET_I2CFREQ(100);   // set I2C SCL frequency to 100kHz
	NETTOUT = 10;       // network transmitt timeout 10 * 10 = 100ms

	sensor_initialized = init_ds1621(DS1621_ADDR, 23.5, 29.5);

    SET_TIMER(1000);
    TIMER_NONBLOCKING;
	while(1)
	{
	    if (FNET) {
            cmd = NETRXBUF[0];
            switch (cmd) {
                case 0x10 : ptemp = (FL_INT_T*)&NETRXBUF[1];
                            ds1621_set_TL(DS1621_ADDR, ftemp_to_u16(ptemp->fl));

                            if (ds1621_get_TL(DS1621_ADDR, &tmp16)) {
                                packet.cmd = 0x20;
                                packet.temp.fl = uitemp_to_float(tmp16);
                                net_sendbytes((uint8_t*)(&packet),5, CH_TCP);
                            }
                            break;

                case 0x11 : ptemp = (FL_INT_T*)&NETRXBUF[1];
                            ds1621_set_TH(DS1621_ADDR, ftemp_to_u16(ptemp->fl));

                            if (ds1621_get_TH(DS1621_ADDR, &tmp16)) {
                                packet.cmd = 0x21;
                                packet.temp.fl = uitemp_to_float(tmp16);
                                net_sendbytes((uint8_t*)(&packet),5, CH_TCP);
                            }
                            break;

                case 0x20 : if (ds1621_get_TL(DS1621_ADDR, &tmp16)) {
                                packet.cmd = 0x20;
                                packet.temp.fl = uitemp_to_float(tmp16);
                                net_sendbytes((uint8_t*)(&packet),5, CH_TCP);
                            }
                            break;
                case 0x21 : if (ds1621_get_TH(DS1621_ADDR, &tmp16)) {
                                packet.cmd = 0x21;
                                packet.temp.fl = uitemp_to_float(tmp16);
                                net_sendbytes((uint8_t*)(&packet),5, CH_TCP);
                            }
                            break;
            }
            FNET = 0;
	    }

        if (FTMR) {     // check if timer flag is set
            FTMR = 0;   // Clear timer flag
            if (sensor_initialized == 0) {
                sensor_initialized = ds1621_start_convert(DS1621_ADDR);
            } else {
                if (ds1621_read_hrtemp(DS1621_ADDR, &packet.temp.fl)) {
                    if (ds1621_get_config(DS1621_ADDR, &config)) {
                        if(config & (DS1621_THF | DS1621_TLF)) {
                                ds1621_reset_flags(DS1621_ADDR);
                        }
                    }
                    if (FEST1) {  // transmit if connection is established
                        packet.cmd = 0xAA;
                        packet.config = config;
                        net_sendbytes((uint8_t*)(&packet),6, CH_TCP);
                    }
                }
            }
        }
    }
}
