/*-------------------------------------------------------------------------
   i2c.h: I2C Library for the Virtual Machine 8051 / LAN I2C Adapter VM

   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   -------------------------------------------------------------------------------
   Date:    2022-07-19
   Version: 1.00
   Comment: Initial release
----------------------------------------------------------------------------------*/
#ifndef I2C_H
#define I2C_H

//----------I2C Result Code--------
#define I2C_RESULT_OK           0x00	// Requested Request was executed successfully.
#define ERR_I2C_ERROR        	0x01	// Unknown error condition.
#define ERR_I2C_ARBLOST      	0x02	// Arbitration lost.
#define ERR_I2C_BUS_ERROR     	0x03	// I2C bus error
#define ERR_I2C_INVALID_WLEN    0x04	// Invalid write / read data length.
#define ERR_I2C_INVALID_RLEN    0x05	// Invalid write / read data length.
#define ERR_I2C_NAK_WADDR      	0x06	// No acknowledgement received from slave during WRITE address phase.
#define ERR_I2C_NAK_RADDR      	0x07	// No acknowledgement received from slave during READ address phase.
#define ERR_I2C_NAK_DAT       	0x08	// No acknowledgement received from slave during WRITE data phase.
#define ERR_I2C_TIMEOUT       	0x09	// Time-Out.
#define ERR_I2C_BUSY          	0x0A	// I2C transmistter is busy.
#define ERR_I2C_UNKNOWN         0xFF    // Unknown Error Code

#define SET_I2CFREQ(x)  (I2CFREQ = ((uint16_t)(x)) / 5)  // SCL frequency - 5kHz steps
#define GET_I2CFREQ     (((uint16_t) I2CFREQ) * 5)

#if (USE_REPEATED_START == 1)
    #define REPEATED_START  1
#else
    #define REPEATED_START  0
#endif

#define I2C_WRITE                   (I2CCON = I2CWR)
#define I2C_READ                    (I2CCON = I2CRD)
#define I2C_WRITE_READ(rep_start)   (rep_start ? (I2CCON = (I2CWR | I2CRD | I2CRS)) : (I2CCON = (I2CWR | I2CRD)))

#if (USE_I2C_LIBRARY == 1)
extern uint8_t i2c_checkdevice(uint8_t slvaddr);

#if (USE_I2C_WRITE == 1)
extern uint8_t i2c_write(uint8_t slvaddr, uint8_t * dat, uint8_t len);
#endif
#if (USE_I2C_WRITE_8BIT == 1)
extern uint8_t i2c_write_8bit(uint8_t slvaddr, uint8_t dat);
#endif
#if (USE_I2C_WRITE_16BIT == 1)
extern uint8_t i2c_write_16bit(uint8_t slvaddr, uint16_t dat);
#endif
#if (USE_I2C_WRITE_32BIT == 1)
extern uint8_t i2c_write_32bit(uint8_t slvaddr, uint32_t dat);
#endif
#if (USE_I2C_READ == 1)
extern uint8_t i2c_read(uint8_t slvaddr, uint8_t * dat, uint8_t * len);
#endif
#if (USE_I2C_READ_8BIT == 1)
extern uint8_t i2c_read_8bit(uint8_t slvaddr, uint8_t * dat);
#endif
#if (USE_I2C_READ_16BIT == 1)
extern uint8_t i2c_read_16bit(uint8_t slvaddr, uint16_t * dat);
#endif
#if (USE_I2C_READ_32BIT == 1)
extern uint8_t i2c_read_32bit(uint8_t slvaddr, uint32_t * dat);
#endif
#if (USE_I2C_WR8_RD8 == 1)
extern uint8_t i2c_wr8_rd8(uint8_t slvaddr, uint8_t wdat, uint8_t * rdat);
#endif
#if (USE_I2C_WR8_RD16 == 1)
extern uint8_t i2c_wr8_rd16(uint8_t slvaddr, uint8_t wdat, uint16_t * rdat);
#endif
#if (USE_I2C_WR8_RD32 == 1)
extern uint8_t i2c_wr8_rd32(uint8_t slvaddr, uint8_t wdat, uint32_t * rdat);
#endif
#if (USE_I2C_WR16_RD8 == 1)
extern uint8_t i2c_wr16_rd8(uint8_t slvaddr, uint16_t wdat, uint8_t * rdat);
#endif
#if (USE_I2C_WR16_RD16 == 1)
extern uint8_t i2c_wr16_rd16(uint8_t slvaddr, uint16_t wdat, uint16_t * rdat);
#endif
#if (USE_I2C_WR16_RD32 == 1)
extern uint8_t i2c_wr16_rd32(uint8_t slvaddr, uint16_t wdat, uint32_t * rdat);
#endif
#if (USE_I2C_REG8_WR8 == 1)
extern uint8_t i2c_reg8_wr8(uint8_t slvaddr, uint8_t reg, uint8_t dat);
#endif
#if (USE_I2C_REG8_WR16 == 1)
extern uint8_t i2c_reg8_wr16(uint8_t slvaddr, uint8_t reg, uint16_t dat);
#endif
#if (USE_I2C_REG8_WR32 == 1)
extern uint8_t i2c_reg8_wr32(uint8_t slvaddr, uint8_t reg, uint32_t dat);
#endif
#if (USE_I2C_REG16_WR8 == 1)
extern uint8_t i2c_reg16_wr8(uint8_t slvaddr, uint16_t reg, uint8_t dat);
#endif
#if (USE_I2C_REG16_WR16 == 1)
extern uint8_t i2c_reg16_wr16(uint8_t slvaddr, uint16_t reg, uint16_t dat);
#endif
#if (USE_I2C_REG16_WR32 == 1)
extern uint8_t i2c_reg16_wr32(uint8_t slvaddr, uint16_t reg, uint32_t dat);
#endif
#if (USE_I2C_REG8_RD8 == 1)
extern uint8_t i2c_reg8_rd8(uint8_t slvaddr, uint8_t reg, uint8_t * rdat);
#endif
#if (USE_I2C_REG8_RD16 == 1)
extern uint8_t i2c_reg8_rd16(uint8_t slvaddr, uint8_t reg, uint16_t * rdat);
#endif
#if (USE_I2C_REG8_RD32 == 1)
extern uint8_t i2c_reg8_rd32(uint8_t slvaddr, uint8_t reg, uint32_t * rdat);
#endif
#if (USE_I2C_REG16_RD8 == 1)
extern uint8_t i2c_reg16_rd8(uint8_t slvaddr, uint16_t reg, uint8_t * rdat);
#endif
#if (USE_I2C_REG16_RD16 == 1)
extern uint8_t i2c_reg16_rd16(uint8_t slvaddr, uint16_t reg, uint16_t * rdat);
#endif
#if (USE_I2C_REG16_RD32 == 1)
extern uint8_t i2c_reg16_rd32(uint8_t slvaddr, uint16_t reg, uint32_t * rdat);
#endif



#endif

#endif
