/*-------------------------------------------------------------------------
   net.h: Network Library for the Virtual Machine 8051 / LAN I2C Adapter VM

   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   -------------------------------------------------------------------------------
   Date:    2022-07-19
   Version: 1.00
   Comment: Initial release
----------------------------------------------------------------------------------*/


#ifndef NET_H
#define NET_H

#ifndef VMDEVICE_H_INCLUDED
    #define CH_TCP              0x01
    #define CH_UDP              0x02
    #define CH_VMTCP            0x03
    #define CH_VMUDP            0x04

    #define DEBUG_SOCKET_NUM    CH_VMUDP
    #define DEBUG_IP            0
    #define DEBUG_PORT          0
    #define DEBUG_NO_HEADER     1
#endif

#define NET_RESULT_OK           0x00    // data transmission OK
#define NET_TRANSMIT_PENDING    0x01	// data transmission pending
#define ERR_NET_SOCKNUM     	0x02    // Invalid socket number
#define ERR_NET_SOCKINIT        0x03    // Socket is not initialized or SIPR is Zero IP address when Sn_MR_TCP
#define ERR_NET_SOCKCLOSED      0x04    // Socket unexpectedly closed.
#define ERR_NET_SOCKMODE        0x05    // Invalid socket mode for socket operation.
#define ERR_NET_SOCKSTATUS      0x06    // Invalid socket status for socket operation.
#define ERR_NET_PORTZERO        0x07    // Port number is zero
#define ERR_NET_IPINVALID       0x08    // Invalid IP address
#define ERR_NET_TIMEOUT         0x09    // Timeout occurred
#define ERR_NET_DATALEN         0x0A    // Data length is zero or greater than buffer max size.
#define ERR_NET_BUFFER          0x0B    // Socket buffer is not enough for data communication.
#define ERR_NET_BUSY        	0x0C    // Socket buffer is not enough for data communication.
#define ERR_NET_ACK_TIMEOUT		0x0D    // No ACK, Timeout
#define ERR_NET_UNKNOWN         0xFF    // Unknown Error Code

#define NET_MAX_SIZE    255

#if (USE_NET_SENDTXT == 1)
extern uint16_t net_sendtxt(const char * stxt, uint8_t socket);
#endif
#if (USE_NET_SENDTXT_TO == 1)
extern uint16_t net_sendtxt_to(const char * stxt, uint32_t ip, uint16_t port, uint8_t noheader);
#endif
#if (USE_NET_SENDBYTES == 1)
extern uint8_t net_sendbytes(uint8_t * data, uint8_t ucsize, uint8_t socket);
#endif
#if (USE_NET_SENDBYTES_TO == 1)
extern uint8_t net_sendbytes_to(uint8_t * data, uint8_t ucsize, , uint32_t ip, uint16_t port, uint8_t noheader);
#endif

#ifdef DEBUG_PRINT

#if (USE_PRINT_U8 == 1)
extern void print_u8(const char * stxt, uint8_t value);
#endif

#if (USE_PRINT_U16 == 1)
extern void print_u16(const char * stxt, uint16_t value);
#endif

#if (USE_PRINT_FLOAT == 1)
extern void print_float(const float value);
#endif

#endif

#endif
