/*-------------------------------------------------------------------------
   VM8051.h: Register Declarations for the Virtual Machine 8051 / LAN I2C Adapter VM

   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   -------------------------------------------------------------------------------
   Date:    2022-07-19
   Version: 1.00
   Comment: Initial release
----------------------------------------------------------------------------------*/

#ifndef VM8051_H
#define VM8051_H

/*  BYTE Register  */
__sfr __at (0x81) SP        ;
__sfr __at (0x82) DPL       ;
__sfr __at (0x83) DPH       ;
__sfr __at (0xD0) PSW       ;
__sfr __at (0xE0) ACC       ;
__sfr __at (0xF0) B         ;

__sfr __at (0xC0) IE2       ;
__sfr __at (0xC1) I2CCON    ;
__sfr __at (0xC2) SLVADDR   ;
__sfr __at (0xC3) I2CFREQ   ;
__sfr __at (0xC4) I2CTXLEN  ;
__sfr __at (0xC5) I2CRXLEN  ;
__sfr __at (0xC6) I2CSTAT   ;
__sfr __at (0xC7) EXTCON    ;
__sfr __at (0xC8) IFLAG     ;
__sfr __at (0xC9) TMRCON    ;
__sfr __at (0xCA) TMRL      ;
__sfr __at (0xCB) TMRH      ;
__sfr16 __at (0xCBCA) TMR   ;

__sfr __at (0xA9) TXIP0     ;
__sfr __at (0xAA) TXIP1     ;
__sfr __at (0xAB) TXIP2     ;
__sfr __at (0xAC) TXIP3     ;
__sfr32 __at (0xACABAAA9) TXIP;

__sfr __at (0xAD) TXPORTL   ;
__sfr __at (0xAE) TXPORTH   ;
__sfr16 __at (0xAEAD) TXPORT;

__sfr __at (0xAF) TXCH      ;
__sfr __at (0xB1) RXIP0     ;
__sfr __at (0xB2) RXIP1     ;
__sfr __at (0xB3) RXIP2     ;
__sfr __at (0xB4) RXIP3     ;
__sfr32 __at (0xB4B3B2B1) RXIP;

__sfr __at (0xB5) RXPORTL   ;
__sfr __at (0xB6) RXPORTH   ;
__sfr16 __at (0xB6B5) RXPORT;

__sfr __at (0xB7) RXCH      ;
__sfr __at (0xB9) NETCON    ;
__sfr __at (0xBA) NETTXLEN  ;
__sfr __at (0xBB) NETRXLEN  ;
__sfr __at (0xBC) NETSTAT   ;
__sfr __at (0xBD) NETTOUT   ;


/*  BIT Register  */
/*  IE2   */
__sbit __at (0xC0) ENET     ;
__sbit __at (0xC1) ETMR     ;
__sbit __at (0xC2) EEXT     ;

/*  IFLAG   */
__sbit __at (0xC8) FNET     ;
__sbit __at (0xC9) FTMR     ;
__sbit __at (0xCA) FEXT     ;

/*  PSW   */
__sbit __at (0xD0) P        ;
__sbit __at (0xD1) F1       ;
__sbit __at (0xD2) OV       ;
__sbit __at (0xD3) RS0      ;
__sbit __at (0xD4) RS1      ;
__sbit __at (0xD5) F0       ;
__sbit __at (0xD6) AC       ;
__sbit __at (0xD7) CY       ;

volatile __xdata __at (0x0800) unsigned char I2CTXBUF[255];
volatile __xdata __at (0x0900) unsigned char I2CRXBUF[255];
volatile __xdata __at (0x0A00) unsigned char NETTXBUF[255];
volatile __xdata __at (0x0B00) unsigned char NETRXBUF[255];

/* Interrupt numbers: address = (number * 8) + 3 */
#define IE0_VECTOR      0       /* 0x03 external interrupt 0 */
#define TF0_VECTOR      1       /* 0x0b timer 0 */
#define IE1_VECTOR      2       /* 0x13 external interrupt 1 */
#define TF1_VECTOR      3       /* 0x1b timer 1 */
#define SI0_VECTOR      4       /* 0x23 serial port 0 */
#define NET_VECTOR      5       /* 0x2B NETWORK */
#define TMR_VECTOR      6       /* 0x33 TIMER */
#define EXT_VECTOR      7       /* 0x3B VM external interrupt */

#define NETWR           0x01    // start network data transmission (TCP or UDP)
#define NETNH           0x02    // no header option
#define FPHY            0x10    // network PHY STATUS flag (read only)
#define FEST1           0x20    // TCP/IP socket 1 established flag (read only)
#define FEST3           0x40    // VM TCP/IP socket 3 established flag (read only)

#define I2CWR           0x01    // I2C write
#define I2CRD           0x02    // I2C read
#define I2CRS           0x04    // I2C write and read with repeated start

#define IE_NET          0x01
#define IE_TMR          0x02
#define IE_EXT          0x04

#define IF_NET          0x01
#define IF_TMR          0x02
#define IF_EXT          0x04

#define TMRRN           0x01
#define TMRBL           0x02
#define TMRRS           0x04

#define EXT_DISABLE		0x00
#define EXT_ENABLE		0x01
#define EXT_FALLING		0x00
#define EXT_RISING		0x02

#define ENABLE_NETWORK_INTERRUPT    (ENET = 1)
#define ENABLE_TIMER_INTERRUPT      (ETMR = 1)
#define ENABLE_EXTERNAL_INTERRUPT   (EEXT = 1)
#define DISABLE_NETWORK_INTERRUPT   (ENET = 0)
#define DISABLE_TIMER_INTERRUPT     (ETMR = 0)
#define DISABLE_EXTERNAL_INTERRUPT  (EEXT = 0)

#define NETWORK_WRITE               (NETCON = NETWR)
#define NETWORK_WRITE_NO_HEADER     (NETCON = NETWR | NETNH)

#define TIMER_STOP                  (TMRCON = 0)
#define TIMER_BLOCKING_ONESHOT      (TMRCON = TMRRN | TMRBL)
#define TIMER_NONBLOCKING_ONESHOT   (TMRCON = TMRRN)
#define TIMER_NONBLOCKING           (TMRCON = TMRRN | TMRRS)
#define SET_TIMER(x)	            (TMRL = ((x) & 0x00FF)); (TMRH = (((x) >> 8) & 0x00FF))

#define EXTERNAL_INTERRUPT_FALLING  (EXTCON = EXT_ENABLE | EXT_FALLING)
#define EXTERNAL_INTERRUPT_RISING   (EXTCON = EXT_ENABLE | EXT_RISING)
#define DISABLE_INTERRUPT_INPUT     (EXTCON = EXT_DISABLE)

#endif
