#include "vmdevice.h"
#include <stdint.h>
#include "i2c.h"

#if (USE_I2C_LIBRARY == 1)

#if (USE_I2C_CHECK == 1)
uint8_t i2c_checkdevice(uint8_t slvaddr)
{
    SLVADDR = slvaddr;                  // load slave addr
    I2CTXLEN = 0;                       // set to zero, we do want to write any data
    I2C_WRITE;                          // start transmission, blocking function
    return(I2CSTAT == I2C_RESULT_OK);   // returns 1 if I2C device is available
}
#endif

#if (USE_I2C_WRITE == 1)
uint8_t i2c_write(uint8_t slvaddr, uint8_t * dat, uint8_t len)
{
    uint8_t ii;

    SLVADDR = slvaddr;              // load slave addr
    for(ii=0;ii<len;ii++)
    {
        I2CTXBUF[ii] = dat[ii];
    }
    I2CTXLEN = len;                 // set number of bytes to transmit
    I2C_WRITE;                      // start transmission, blocking function
//    *byteswritten = I2CTXLEN;     // I2CTXLEN returns number of bytes were written to the I2C bus
    return(I2CSTAT);                // 0x00 if successful, else error. See possible error codes
}
#endif

#if (USE_I2C_WRITE_8BIT == 1)
uint8_t i2c_write_8bit(uint8_t slvaddr, uint8_t dat)
{
    SLVADDR     = slvaddr;  // load slave addr
    I2CTXBUF[0] = dat;      // load data byte
    I2CTXLEN    = 1;        // set number of bytes to transmit
    I2C_WRITE;              // start transmission, blocking function
    return(I2CSTAT);        // 0x00 if successful, else error. See possible error codes
}
#endif

#if (USE_I2C_WRITE_16BIT == 1)
uint8_t i2c_write_16bit(uint8_t slvaddr, uint16_t dat)
{
    SLVADDR     = slvaddr;              // load slave addr
#if (I2CWR_MSB_FIRST == 1)
    I2CTXBUF[0] = (uint8_t) (dat >> 8); // load MSB
    I2CTXBUF[1] = (uint8_t) dat;        // load LSB
#else
    I2CTXBUF[0] = (uint8_t) dat;        // load LSB
    I2CTXBUF[1] = (uint8_t) (dat >> 8); // load MSB
#endif
    I2CTXLEN    = 2;                    // set number of bytes to transmit
    I2C_WRITE;                          // start transmission, blocking function
    return(I2CSTAT);                    // 0x00 if successful, else error. See possible error codes
}
#endif

#if (USE_I2C_WRITE_32BIT == 1)
uint8_t i2c_write_32bit(uint8_t slvaddr, uint32_t dat)
{
    SLVADDR     = slvaddr;              // load slave addr
#if (I2CWR_MSB_FIRST == 1)
    I2CTXBUF[0] = (uint8_t) (dat >> 24);// load MSB
    I2CTXBUF[1] = (uint8_t) (dat >> 16);
    I2CTXBUF[2] = (uint8_t) (dat >> 8);
    I2CTXBUF[3] = (uint8_t) dat;        // load LSB
#else
    I2CTXBUF[0] = (uint8_t) dat;        // load LSB
    I2CTXBUF[1] = (uint8_t) (dat >> 8);
    I2CTXBUF[2] = (uint8_t) (dat >> 16);
    I2CTXBUF[3] = (uint8_t) (dat >> 24);// load MSB
#endif
    I2CTXLEN    = 4;                    // set number of bytes to transmit
    I2C_WRITE;                          // start transmission, blocking function
    return(I2CSTAT);                    // 0x00 if successful, else error. See possible error codes
}
#endif

#if (USE_I2C_READ == 1)
uint8_t i2c_read(uint8_t slvaddr, uint8_t * dat, uint8_t * len)
{
    uint8_t ii;

    SLVADDR     = slvaddr;              // load slave addr
    I2CRXLEN    = *len;                 // set number of bytes to read
    I2C_READ;                           // start reading, blocking function
    if (I2CSTAT == I2C_RESULT_OK)
    {
        for(ii=0;ii<I2CRXLEN;ii++)
        {
            *dat = I2CRXBUF[ii];
            dat++;
        }
    }
    *len = I2CRXLEN;                // return the number bytes read
    return(I2CSTAT);              // 0x00 if successful, else error. See possible error codes
}
#endif

#if (USE_I2C_READ_8BIT == 1)
uint8_t i2c_read_8bit(uint8_t slvaddr, uint8_t * dat)
{
    SLVADDR     = slvaddr;              // load slave addr
    I2CRXLEN    = 1;                    // set number of bytes to read
    I2C_READ;                           // start reading, blocking function

    if (I2CSTAT == I2C_RESULT_OK)
    {
        *dat = I2CRXBUF[0];
    }
    return(I2CSTAT);
}
#endif

#if (USE_I2C_READ_16BIT == 1)
uint8_t i2c_read_16bit(uint8_t slvaddr, uint16_t * dat)
{
    SLVADDR     = slvaddr;              // load slave addr
    I2CRXLEN    = 2;                    // set number of bytes to read
    I2C_READ;                           // start reading, blocking function

    if (I2CSTAT == I2C_RESULT_OK)
    {
#if (I2CRD_MSB_FIRST == 1)
        *dat = I2CRXBUF[0];
        *dat = (*dat << 8);
        *dat |= I2CRXBUF[1];
#else
        *dat = I2CRXBUF[1];
        *dat = (*dat << 8);
        *dat |= I2CRXBUF[0];
#endif
    }
    return(I2CSTAT);
}
#endif

#if (USE_I2C_READ_32BIT == 1)
uint8_t i2c_read_32bit(uint8_t slvaddr, uint32_t * dat)
{
    SLVADDR     = slvaddr;              // load slave addr
    I2CRXLEN    = 4;                    // set number of bytes to read
    I2C_READ;                           // start reading, blocking function

    if (I2CSTAT == I2C_RESULT_OK)
    {
#if (I2CRD_MSB_FIRST == 1)
        *dat = I2CRXBUF[0];
        *dat = (*dat << 8);
        *dat |= I2CRXBUF[1];
        *dat = (*dat << 8);
        *dat |= I2CRXBUF[2];
        *dat = (*dat << 8);
        *dat |= I2CRXBUF[3];
#else
        *dat = I2CRXBUF[3];
        *dat = (*dat << 8);
        *dat |= I2CRXBUF[2];
        *dat = (*dat << 8);
        *dat |= I2CRXBUF[1];
        *dat = (*dat << 8);
        *dat |= I2CRXBUF[0];
#endif
    }
    return(I2CSTAT);
}
#endif

#if (USE_I2C_WR8_RD8 == 1)
uint8_t i2c_wr8_rd8(uint8_t slvaddr, uint8_t wdat, uint8_t * rdat)
{
    SLVADDR     = slvaddr;          // load slave addr
    I2CTXBUF[0] = wdat;             // load data byte
    I2CTXLEN    = 1;                // set number of bytes to write
    I2CRXLEN    = 1;                // set number of bytes to read
    I2C_WRITE_READ(REPEATED_START); // start write/read, blocking function
    if (I2CSTAT == I2C_RESULT_OK)
    {
        *rdat = I2CRXBUF[0];
    }
    return(I2CSTAT);              // 0x00 if successful, else error. See possible error codes
}
#endif

#if (USE_I2C_WR8_RD16 == 1)
uint8_t i2c_wr8_rd16(uint8_t slvaddr, uint8_t wdat, uint16_t * rdat)
{
    SLVADDR     = slvaddr;          // load slave addr
    I2CTXBUF[0] = wdat;             // load data byte
    I2CTXLEN    = 1;                // set number of bytes to write
    I2CRXLEN    = 2;                // set number of bytes to read
    I2C_WRITE_READ(REPEATED_START); // start write/read, blocking function
    if (I2CSTAT == I2C_RESULT_OK)
    {
#if (I2CRD_MSB_FIRST == 1)
        *rdat = I2CRXBUF[0];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[1];
#else
        *rdat = I2CRXBUF[1];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[0];
#endif
    }
    return(I2CSTAT);              // 0x00 if successful, else error. See possible error codes
}
#endif

#if (USE_I2C_WR8_RD32 == 1)
uint8_t i2c_wr8_rd32(uint8_t slvaddr, uint8_t wdat, uint32_t * rdat)
{
    SLVADDR     = slvaddr;          // load slave addr
    I2CTXBUF[0] = wdat;             // load data byte
    I2CTXLEN    = 1;                // set number of bytes to write
    I2CRXLEN    = 4;                // set number of bytes to read
    I2C_WRITE_READ(REPEATED_START); // start write/read, blocking function
    if (I2CSTAT == I2C_RESULT_OK)
    {
#if (I2CRD_MSB_FIRST == 1)
        *rdat = I2CRXBUF[0];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[1];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[2];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[3];
#else
        *rdat = I2CRXBUF[3];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[2];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[1];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[0];
#endif
    }
    return(I2CSTAT);              // 0x00 if successful, else error. See possible error codes
}
#endif

#if (USE_I2C_WR16_RD8 == 1)
uint8_t i2c_wr16_rd8(uint8_t slvaddr, uint16_t wdat, uint8_t * rdat)
{
    SLVADDR     = slvaddr;                  // load slave addr
#if (I2CWR_MSB_FIRST == 1)
    I2CTXBUF[0] = (uint8_t) (wdat >> 8);    // load MSB
    I2CTXBUF[1] = (uint8_t) wdat;           // load LSB
#else
    I2CTXBUF[0] = (uint8_t) wdat;           // load LSB
    I2CTXBUF[1] = (uint8_t) (wdat >> 8);    // load MSB
#endif
    I2CTXLEN    = 2;                        // set number of bytes to write
    I2CRXLEN    = 1;                        // set number of bytes to read
    I2C_WRITE_READ(REPEATED_START);         // start write/read, blocking function
    if (I2CSTAT == I2C_RESULT_OK)
    {
        *rdat = I2CRXBUF[0];
    }
    return(I2CSTAT);              // 0x00 if successful, else error. See possible error codes
}
#endif

#if (USE_I2C_WR16_RD16 == 1)
uint8_t i2c_wr16_rd16(uint8_t slvaddr, uint16_t wdat, uint16_t * rdat)
{
    SLVADDR     = slvaddr;                  // load slave addr
#if (I2CWR_MSB_FIRST == 1)
    I2CTXBUF[0] = (uint8_t) (wdat >> 8);    // load MSB
    I2CTXBUF[1] = (uint8_t) wdat;           // load LSB
#else
    I2CTXBUF[0] = (uint8_t) wdat;           // load LSB
    I2CTXBUF[1] = (uint8_t) (wdat >> 8);    // load MSB
#endif
    I2CTXLEN    = 2;                        // set number of bytes to write
    I2CRXLEN    = 2;                        // set number of bytes to read
    I2C_WRITE_READ(REPEATED_START);         // start write/read, blocking function
    if (I2CSTAT == I2C_RESULT_OK)
    {
#if (I2CRD_MSB_FIRST == 1)
        *rdat = I2CRXBUF[0];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[1];
#else
        *rdat = I2CRXBUF[1];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[0];
#endif
    }
    return(I2CSTAT);              // 0x00 if successful, else error. See possible error codes
}
#endif

#if (USE_I2C_WR16_RD32 == 1)
uint8_t i2c_wr16_rd32(uint8_t slvaddr, uint16_t wdat, uint32_t * rdat)
{
    SLVADDR     = slvaddr;                  // load slave addr
#if (I2CWR_MSB_FIRST == 1)
    I2CTXBUF[0] = (uint8_t) (wdat >> 8);    // load MSB
    I2CTXBUF[1] = (uint8_t) wdat;           // load LSB
#else
    I2CTXBUF[0] = (uint8_t) wdat;           // load LSB
    I2CTXBUF[1] = (uint8_t) (wdat >> 8);    // load MSB
#endif
    I2CTXLEN    = 2;                        // set number of bytes to write
    I2CRXLEN    = 4;                        // set number of bytes to read
    I2C_WRITE_READ(REPEATED_START);         // start write/read, blocking function
    if (I2CSTAT == I2C_RESULT_OK)
    {
#if (I2CRD_MSB_FIRST == 1)
        *rdat = I2CRXBUF[0];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[1];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[2];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[3];
#else
        *rdat = I2CRXBUF[3];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[2];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[1];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[0];
#endif
    }
    return(I2CSTAT);              // 0x00 if successful, else error. See possible error codes
}
#endif

#if (USE_I2C_REG8_WR8 == 1)
uint8_t i2c_reg8_wr8(uint8_t slvaddr, uint8_t reg, uint8_t dat)
{
    SLVADDR     = slvaddr;  // load slave addr
    I2CTXBUF[0] = reg;      // load reg
    I2CTXBUF[1] = dat;      // load data byte
    I2CTXLEN    = 2;        // set number of bytes to transmit
    I2C_WRITE;              // start transmission, blocking function
    return(I2CSTAT);        // 0x00 if successful, else error. See possible error codes
}
#endif

#if (USE_I2C_REG8_WR16 == 1)
uint8_t i2c_reg8_wr16(uint8_t slvaddr, uint8_t reg, uint16_t dat)
{
    SLVADDR     = slvaddr;              // load slave addr
    I2CTXBUF[0] = reg;                  // load reg
#if (I2CWR_MSB_FIRST == 1)
    I2CTXBUF[1] = (uint8_t) (dat >> 8); // load MSB
    I2CTXBUF[2] = (uint8_t) dat;        // load LSB
#else
    I2CTXBUF[1] = (uint8_t) dat;        // load LSB
    I2CTXBUF[2] = (uint8_t) (dat >> 8); // load MSB
#endif
    I2CTXLEN    = 3;                    // set number of bytes to transmit
    I2C_WRITE;                          // start transmission, blocking function
    return(I2CSTAT);                    // 0x00 if successful, else error. See possible error codes
}
#endif

#if (USE_I2C_REG8_WR32 == 1)
uint8_t i2c_reg8_wr32(uint8_t slvaddr, uint8_t reg, uint32_t dat)
{
    SLVADDR     = slvaddr;              // load slave addr
    I2CTXBUF[0] = reg;                  // load reg
#if (I2CWR_MSB_FIRST == 1)
    I2CTXBUF[1] = (uint8_t) (dat >> 24);// load MSB
    I2CTXBUF[2] = (uint8_t) (dat >> 16);
    I2CTXBUF[3] = (uint8_t) (dat >> 8);
    I2CTXBUF[4] = (uint8_t) dat;        // load LSB
#else
    I2CTXBUF[1] = (uint8_t) dat;        // load LSB
    I2CTXBUF[2] = (uint8_t) (dat >> 8);
    I2CTXBUF[3] = (uint8_t) (dat >> 16);
    I2CTXBUF[4] = (uint8_t) (dat >> 24);// load MSB
#endif
    I2CTXLEN    = 5;                    // set number of bytes to transmit
    I2C_WRITE;                          // start transmission, blocking function
    return(I2CSTAT);                    // 0x00 if successful, else error. See possible error codes
}
#endif

#if (USE_I2C_REG16_WR8 == 1)
uint8_t i2c_reg16_wr8(uint8_t slvaddr, uint16_t reg, uint8_t dat)
{
    SLVADDR     = slvaddr;              // load slave addr
#if (I2CWR_MSB_FIRST == 1)
    I2CTXBUF[0] = (uint8_t) (reg >> 8); // load MSB
    I2CTXBUF[1] = (uint8_t) reg;        // load LSB
#else
    I2CTXBUF[0] = (uint8_t) reg;        // load LSB
    I2CTXBUF[1] = (uint8_t) (reg >> 8); // load MSB
#endif
    I2CTXBUF[2] = dat;                  // load data byte
    I2CTXLEN    = 3;                    // set number of bytes to transmit
    I2C_WRITE;                          // start transmission, blocking function
    return(I2CSTAT);                    // 0x00 if successful, else error. See possible error codes
}
#endif

#if (USE_I2C_REG16_WR16 == 1)
uint8_t i2c_reg16_wr16(uint8_t slvaddr, uint16_t reg, uint16_t dat)
{
    SLVADDR     = slvaddr;              // load slave addr
#if (I2CWR_MSB_FIRST == 1)
    I2CTXBUF[0] = (uint8_t) (reg >> 8); // load MSB
    I2CTXBUF[1] = (uint8_t) reg;        // load LSB
    I2CTXBUF[2] = (uint8_t) (dat >> 8); // load MSB
    I2CTXBUF[3] = (uint8_t) dat;        // load LSB
#else
    I2CTXBUF[0] = (uint8_t) reg;        // load LSB
    I2CTXBUF[1] = (uint8_t) (reg >> 8); // load MSB
    I2CTXBUF[2] = (uint8_t) dat;        // load LSB
    I2CTXBUF[3] = (uint8_t) (dat >> 8); // load MSB
#endif
    I2CTXLEN    = 4;                    // set number of bytes to transmit
    I2C_WRITE;                          // start transmission, blocking function
    return(I2CSTAT);                    // 0x00 if successful, else error. See possible error codes
}
#endif

#if (USE_I2C_REG16_WR32 == 1)
uint8_t i2c_reg16_wr32(uint8_t slvaddr, uint16_t reg, uint32_t dat)
{
    SLVADDR     = slvaddr;              // load slave addr
#if (I2CWR_MSB_FIRST == 1)
    I2CTXBUF[0] = (uint8_t) (reg >> 8); // load MSB
    I2CTXBUF[1] = (uint8_t) reg;        // load LSB
    I2CTXBUF[2] = (uint8_t) (dat >> 24);// load MSB
    I2CTXBUF[3] = (uint8_t) (dat >> 16);
    I2CTXBUF[4] = (uint8_t) (dat >> 8);
    I2CTXBUF[5] = (uint8_t) dat;        // load LSB
#else
    I2CTXBUF[0] = (uint8_t) reg;        // load LSB
    I2CTXBUF[1] = (uint8_t) (reg >> 8); // load MSB
    I2CTXBUF[2] = (uint8_t) dat;        // load LSB
    I2CTXBUF[3] = (uint8_t) (dat >> 8);
    I2CTXBUF[4] = (uint8_t) (dat >> 16);
    I2CTXBUF[5] = (uint8_t) (dat >> 24);// load MSB
#endif
    I2CTXLEN    = 6;                    // set number of bytes to transmit
    I2C_WRITE;                          // start transmission, blocking function
    return(I2CSTAT);                    // 0x00 if successful, else error. See possible error codes
}
#endif

#if (USE_I2C_REG8_RD8 == 1)
uint8_t i2c_reg8_rd8(uint8_t slvaddr, uint8_t reg, uint8_t * rdat)
{
    SLVADDR     = slvaddr;          // load slave addr
    I2CTXBUF[0] = reg;              // load register
    I2CTXLEN    = 1;                // set number of bytes to write
    I2CRXLEN    = 1;                // set number of bytes to read
    I2C_WRITE_READ(REPEATED_START); // start write/read, blocking function
    if (I2CSTAT == I2C_RESULT_OK)
    {
        *rdat = I2CRXBUF[0];
    }
    return(I2CSTAT);                // 0x00 if successful, else error. See possible error codes
}
#endif

#if (USE_I2C_REG8_RD16 == 1)
uint8_t i2c_reg8_rd16(uint8_t slvaddr, uint8_t reg, uint16_t * rdat)
{
    SLVADDR     = slvaddr;          // load slave addr
    I2CTXBUF[0] = reg;              // load register
    I2CTXLEN    = 1;                // set number of bytes to write
    I2CRXLEN    = 2;                // set number of bytes to read
    I2C_WRITE_READ(REPEATED_START); // start write/read, blocking function
    if (I2CSTAT == I2C_RESULT_OK)
    {
#if (I2CRD_MSB_FIRST == 1)
        *rdat = I2CRXBUF[0];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[1];
#else
        *rdat = I2CRXBUF[1];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[0];
#endif
    }
    return(I2CSTAT);                // 0x00 if successful, else error. See possible error codes
}
#endif

#if (USE_I2C_REG8_RD32 == 1)
uint8_t i2c_reg8_rd32(uint8_t slvaddr, uint8_t reg, uint32_t * rdat)
{
    SLVADDR     = slvaddr;          // load slave addr
    I2CTXBUF[0] = reg;              // load register
    I2CTXLEN    = 1;                // set number of bytes to write
    I2CRXLEN    = 4;                // set number of bytes to read
    I2C_WRITE_READ(REPEATED_START); // start write/read, blocking function
    if (I2CSTAT == I2C_RESULT_OK)
    {
#if (I2CRD_MSB_FIRST == 1)
        *rdat = I2CRXBUF[0];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[1];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[2];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[3];
#else
        *rdat = I2CRXBUF[3];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[2];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[1];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[0];
#endif
    }
    return(I2CSTAT);                // 0x00 if successful, else error. See possible error codes
}
#endif

#if (USE_I2C_REG16_RD8 == 1)
uint8_t i2c_reg16_rd8(uint8_t slvaddr, uint16_t reg, uint8_t * rdat)
{
    SLVADDR     = slvaddr;                  // load slave addr
#if (I2CWR_MSB_FIRST == 1)
    I2CTXBUF[0] = (uint8_t) (reg >> 8);     // load reg MSB
    I2CTXBUF[1] = (uint8_t) reg;            // load reg LSB
#else
    I2CTXBUF[0] = (uint8_t) reg;            // load reg LSB
    I2CTXBUF[1] = (uint8_t) (reg >> 8);     // load reg MSB
#endif
    I2CTXLEN    = 2;                        // set number of bytes to write
    I2CRXLEN    = 1;                        // set number of bytes to read
    I2C_WRITE_READ(REPEATED_START);         // start write/read, blocking function
    if (I2CSTAT == I2C_RESULT_OK)
    {
        *rdat = I2CRXBUF[0];
    }
    return(I2CSTAT);                        // 0x00 if successful, else error. See possible error codes
}
#endif

#if (USE_I2C_REG16_RD16 == 1)
uint8_t i2c_reg16_rd16(uint8_t slvaddr, uint16_t reg, uint16_t * rdat)
{
    SLVADDR     = slvaddr;                  // load slave addr
#if (I2CWR_MSB_FIRST == 1)
    I2CTXBUF[0] = (uint8_t) (reg >> 8);     // load reg MSB
    I2CTXBUF[1] = (uint8_t) reg;            // load reg LSB
#else
    I2CTXBUF[0] = (uint8_t) reg;            // load reg LSB
    I2CTXBUF[1] = (uint8_t) (reg >> 8);     // load reg MSB
#endif
    I2CTXLEN    = 2;                        // set number of bytes to write
    I2CRXLEN    = 2;                        // set number of bytes to read
    I2C_WRITE_READ(REPEATED_START);         // start write/read, blocking function
    if (I2CSTAT == I2C_RESULT_OK)
    {
#if (I2CWR_MSB_FIRST == 1)
        *rdat = I2CRXBUF[0];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[1];
#else
        *rdat = I2CRXBUF[1];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[0];
#endif
    }
    return(I2CSTAT);              // 0x00 if successful, else error. See possible error codes
}
#endif

#if (USE_I2C_REG16_RD32 == 1)
uint8_t i2c_reg16_rd32(uint8_t slvaddr, uint16_t reg, uint32_t * rdat)
{
    SLVADDR     = slvaddr;                  // load slave addr
#if (I2CWR_MSB_FIRST == 1)
    I2CTXBUF[0] = (uint8_t) (reg >> 8);     // load reg MSB
    I2CTXBUF[1] = (uint8_t) reg;            // load reg LSB
#else
    I2CTXBUF[0] = (uint8_t) reg;            // load reg LSB
    I2CTXBUF[1] = (uint8_t) (reg >> 8);     // load reg MSB
#endif
    I2CTXLEN    = 2;                        // set number of bytes to write
    I2CRXLEN    = 4;                        // set number of bytes to read
    I2C_WRITE_READ(REPEATED_START);         // start write/read, blocking function
    if (I2CSTAT == I2C_RESULT_OK)
    {
#if (I2CWR_MSB_FIRST == 1)
        *rdat = I2CRXBUF[0];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[1];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[2];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[3];
#else
        *rdat = I2CRXBUF[3];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[2];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[1];
        *rdat = (*rdat << 8);
        *rdat |= I2CRXBUF[0];
#endif
    }
    return(I2CSTAT);              // 0x00 if successful, else error. See possible error codes
}
#endif


#endif
