#include "vmdevice.h"
#include <stdint.h>
#include "i2c.h"
#include "ltc1669.h"

/*
*   The LTC1669 is a WRITE-ONLY device, therefore we need to store values localy
*/
static uint16_t DAC_Value = 0x00;   // Output is set to zero after power on
static uint8_t DAC_Cmd = 0x00;      // Bit 0 => SY = 0 Update on Stop condition only (Power-On Default)
                                    // Bit 1 => SD = 0 Puts the device in standard operating mode (Power-On Default)
                                    // Bit 2 => BG = 0 Selects the supply as the reference (Power-On Default)

uint16_t read_dac_value(void)
{
    return(DAC_Value);
}

uint8_t read_dac_cmd(void)
{
    return(DAC_Cmd);
}

uint8_t write_dac(uint8_t slvaddr, uint8_t cmd, uint16_t value)
{
    uint8_t ret;

    value &= 0x03FF;
    cmd &= 0x07;

    ret = i2c_reg8_wr16(slvaddr, cmd, value);
    if (ret == I2C_RESULT_OK){
        DAC_Value = value;
        DAC_Cmd = cmd;
    }
    return(ret == I2C_RESULT_OK);
}

uint8_t write_dac_value(uint8_t slvaddr, uint16_t value)
{
    return(write_dac(slvaddr, DAC_Cmd, value));
}

uint8_t write_dac_cmd(uint8_t slvaddr, uint8_t cmd)
{
    return(write_dac(slvaddr, cmd, DAC_Value));
}

uint8_t set_dac_cmd_bit(uint8_t slvaddr, uint8_t cmd_bit , uint8_t value)
{
    uint8_t cmd = (DAC_Cmd & (~(1 << cmd_bit))) | ((value & 1) << cmd_bit);
    return(write_dac_cmd(slvaddr, cmd));
}

uint8_t get_dac_cmd_bit(uint8_t cmd_bit)
{
    return((DAC_Cmd >> cmd_bit) & 0x01);
}
