#include "vmdevice.h"
#include <stdint.h>
#include <stdio.h>
#include <math.h>
#include "net.h"

#ifdef DEBUG_PRINT
static __xdata char str[32];

//putchar is needed for printf
//since putchar writes data char by char, printf is too slow
//use sprintf() + net_sendbytes() instead, see below
#if (USE_PUTCHAR == 1)
int putchar(int c)
{
    uint8_t i = 0;

    TXCH   = DEBUG_SOCKET_NUM;
    TXIP   = DEBUG_IP;
    TXPORT = DEBUG_PORT;

    NETTXBUF[0] = c;
    NETTXLEN    = 1;
#if (DEBUG_NO_HEADER == 1)
    NETCON = NETWR | NETNH;
#else
    NETCON = NETWR;
#endif
    while(NETSTAT == NET_TRANSMIT_PENDING);

    return(c);
}
#endif

#if (USE_PRINT_U8 == 1)
void print_u8(const char * stxt, uint8_t value)
{
    sprintf(str,stxt, value);
    net_sendtxt(str, DEBUG_SOCKET_NUM);
}
#endif

#if (USE_PRINT_U16 == 1)
void print_u16(const char * stxt, uint16_t value)
{
    sprintf(str,stxt, value);
    net_sendtxt(str, DEBUG_SOCKET_NUM);
}
#endif

//SDCC does not supoort float sprintf by default
#if (USE_PRINT_FLOAT == 1)
void print_float(const float value)
{

    float fval;
    float ffrac;
    uint16_t uival;
    uint16_t uifrac;
    uint8_t neg_sign = 0;

    if (value < 0) neg_sign = 1;

    fval = ((int16_t)value);
    ffrac = (value - fval) * 1000;

    uival = (uint16_t) fabsf(fval);
    uifrac = (uint16_t) fabsf(ffrac);

    if (neg_sign) {
        sprintf(str,"-%d.%d", uival, uifrac);
    } else {
        sprintf(str,"%d.%d", uival, uifrac);
    }

    net_sendtxt(str,DEBUG_SOCKET_NUM);
}
#endif
#endif

#if (USE_NET_SENDTXT == 1)
uint16_t net_sendtxt(const char * stxt, uint8_t socket)
{
    uint16_t i = 0;
    uint8_t ii = 0;
    uint16_t size_sent = 0;

    TXCH = socket;
    TXIP = 0;
    TXPORT = 0;
    while (stxt[i]) {
        ii = 0;
        while (stxt[i+ii] && (ii < NET_MAX_SIZE)) {
            NETTXBUF[ii] = stxt[i+ii];
            ii++;
        }
        if (ii > 0) {
            NETTXLEN = ii;
            NETCON = NETWR;
            while(NETSTAT == NET_TRANSMIT_PENDING);
            if (NETSTAT != NET_RESULT_OK) break;
            size_sent += NETTXLEN;
        };

        if (ii == 0) break;
        i += ii;
    }
    return(size_sent);
}
#endif
#if (USE_NET_SENDTXT_TO == 1)
uint16_t net_sendtxt_to(const char * stxt, uint32_t ip, uint16_t port, uint8_t noheader)
{
    uint16_t i = 0;
    uint8_t ii = 0;
    uint8_t send_cmd;
    uint16_t size_sent = 0;

    if ((ip == 0) || (port == 0)) return(0);

    TXCH = 0;
    TXIP = ip;
    TXPORT = port;

    if (noheader) {
        send_cmd = NETWR | NETNH;
    } else {
        send_cmd = NETWR;
    }

    while (stxt[i]) {
        ii = 0;
        while (stxt[i+ii] && (ii < NET_MAX_SIZE)) {
            NETTXBUF[ii] = stxt[i+ii];
            ii++;
        }
        if (ii > 0) {
            NETTXLEN = ii;
            NETCON = send_cmd;
            while(NETSTAT == NET_TRANSMIT_PENDING);
            if (NETSTAT != NET_RESULT_OK) break;
            size_sent += NETTXLEN;
        };

        if (ii == 0) break;
        i += ii;
    }
    return(size_sent);
}
#endif
#if (USE_NET_SENDBYTES == 1)
uint8_t net_sendbytes(uint8_t * data, uint8_t ucsize, uint8_t socket)
{
    uint8_t i = 0;

    if (ucsize == 0) return(0);
    if ((socket == 0) || (socket > CH_VMUDP)) return(0);

    TXCH = socket;
    TXIP = 0;
    TXPORT = 0;

    for(i=0; i<ucsize; i++) {
        NETTXBUF[i] = data[i];
    }

    NETTXLEN = ucsize;
    NETCON = NETWR;
    while(NETSTAT == NET_TRANSMIT_PENDING);

    return(NETTXLEN);
}
#endif
#if (USE_NET_SENDBYTES_TO == 1)
uint8_t net_sendbytes_to(uint8_t * data, uint8_t ucsize, , uint32_t ip, uint16_t port, uint8_t noheader)
{
    uint8_t i = 0;
    uint8_t send_cmd;

    if (ucsize == 0) return(0);
    if ((ip == 0) || (port == 0)) return(0);

    TXCH = 0;
    TXIP = ip;
    TXPORT = port;

    if (noheader) {
        send_cmd = NETWR | NETNH;
    } else {
        send_cmd = NETWR;
    }

    for(i=0; i<ucsize; i++) {
        NETTXBUF[i] = data[i];
    }

    NETTXLEN = ucsize;
    NETCON = send_cmd;
    while(NETSTAT == NET_TRANSMIT_PENDING);

    return(NETTXLEN);
}
#endif
